#include "bsp_rtc.h"
#include "ble_conn_state.h"
#include "nrf_ble_scan.h"
#include "app_timer.h"
#include "bsp_battery.h"
#include "drv_mpu9250.h"
#include "nrf_delay.h"

#define COMPARE_COUNTERTIME  (1UL)                                        /**< Get Compare event COMPARE_TIME seconds after the counter starts from 0. */

static const nrf_drv_rtc_t rtc = NRF_DRV_RTC_INSTANCE(2); /**< Declaring an instance of nrf_drv_rtc for RTC2. */

static uint16_t	m_sleeptimes_S = 0;

/** @brief: Function for handling the RTC0 interrupts.
 * Triggered on TICK and COMPARE0 match.
 */
static void rtc_handler(nrf_drv_rtc_int_type_t int_type)
{
	uint32_t err_code;
    if (int_type == NRF_DRV_RTC_INT_COMPARE0)
    {
		nrf_drv_rtc_counter_clear(&rtc);
		err_code = nrf_drv_rtc_cc_set(&rtc,0,m_sleeptimes_S,true);
		APP_ERROR_CHECK(err_code);
        DEBUG_LOG("NRF_DRV_RTC_INT_COMPARE0(0x%X)\n",NRF_RTC0->COUNTER);
    }
    else if (int_type == NRF_DRV_RTC_INT_TICK)
    {
//        DEBUG_LOG("NRF_DRV_RTC_INT_TICK(0x%X)\n",NRF_RTC0->COUNTER);
    }
}

/** @brief Function initialization and configuration of RTC driver instance.
 */
void RTC_init(void)
{
	uint32_t err_code;

	//Initialize RTC instance
	nrf_drv_rtc_config_t config = NRF_DRV_RTC_DEFAULT_CONFIG;
	config.prescaler = 0;// f(RTC) = 32.768kHZ/(prescaler+1) = 8HZ = 125ms
	err_code = nrf_drv_rtc_init(&rtc, &config, rtc_handler);
	APP_ERROR_CHECK(err_code);
	
	err_code = nrf_drv_rtc_cc_set(&rtc,0,300,true);
	APP_ERROR_CHECK(err_code);
	
	nrf_drv_rtc_counter_clear(&rtc);

//	//Enable tick event & interrupt
//	nrf_drv_rtc_tick_enable(&rtc,true);

//	//Set compare channel to trigger interrupt after COMPARE_COUNTERTIME seconds
//	err_code = nrf_drv_rtc_cc_set(&rtc,0,COMPARE_COUNTERTIME * 8,true);
//	APP_ERROR_CHECK(err_code);

//	//Power on RTC instance
	nrf_drv_rtc_enable(&rtc);

}

void rtc_start(uint16_t sleep_S)
{
	uint32_t err_code;
	
	m_sleeptimes_S = sleep_S;
		//Set compare channel to trigger interrupt after sleep_S seconds
	err_code = nrf_drv_rtc_cc_set(&rtc,0,m_sleeptimes_S,true);
	APP_ERROR_CHECK(err_code);
	
	nrf_drv_rtc_counter_clear(&rtc);
	
	nrf_drv_rtc_enable(&rtc);
//	DEBUG_LOG("nrf_drv_rtc_enable(0x%X)\n",NRF_RTC0->COUNTER);
}

void rtc_stop(void)
{
	nrf_drv_rtc_disable(&rtc);
//	DEBUG_LOG("nrf_drv_rtc_disable(0x%X)\n",NRF_RTC0->COUNTER);
}

/*
@brief			˯ߺ
@param			sleep_ms	:˯ʱ䣬units of 125 ms
@return			ʵ˯ʱ
*/
uint32_t rtc_sleep(uint32_t sleep_125ms)
{
	uint32_t sleep_times,wakeup_times,actual_sleeptimes;
	//رɨ
//	if(!ble_conn_state_central_conn_count())//Ϊû²ֹͣɨ
//	{
//		nrf_ble_scan_stop();
////		DEBUG_LOG("scan off...\n");
//	}
	
	//ͣʱ
	
//	app_timer_pause();
//	nrf_delay_ms(100);
	DEBUG_LOG("timer off...\n");
	
	//رADC
//	ADC_Enable(false);
//	DEBUG_LOG("adc off...\n");
	
	//趨rtcʱҿrtc
	rtc_start(sleep_125ms);
//	DEBUG_LOG("rtc start(%d)ms\n",sleep_125ms * 125);
	
	sleep_times = NRF_RTC0->COUNTER;
	
	//˯
	CRITICAL_REGION_ENTER();
	__SEV();
	__WFE();
	CRITICAL_REGION_EXIT();
	(void)sd_app_evt_wait();
	wakeup_times = NRF_RTC0->COUNTER;
//	DEBUG_LOG("===============>rtc sleep_times(%d)\n",sleep_times);
//	DEBUG_LOG("===============>rtc wakeup_times(%d)\n",wakeup_times);
	//ֹͣrtc
	rtc_stop();
//	DEBUG_LOG("rtc end...\n");
	
	//ADC
//	ADC_Enable(true);
//	DEBUG_LOG("adc on...\n");
	
	//ָʱ
//	app_timer_resume();
	DEBUG_LOG("timer on...\n");
	nrf_delay_ms(100);
	actual_sleeptimes = (wakeup_times - sleep_times)/32.768*1000;
	
	return  actual_sleeptimes/1000;
}





