#ifndef __HAL_MINIFDS_H
#define __HAL_MINIFDS_H

#include "system.h"
#include "bsp_flash.h"

#define USE_16BIT_KEY 1
#define MAX_RECORD_LENGTH 0xff

#if USE_16BIT_KEY
#define key_t unsigned short
#define length_t uint8_t
#else 
#define length_t unsigned short
#define key_t uint8_t
#endif

/**@brief   The record metadata as stored in flash.
 * @warning Do not edit or reorder the fields in this structure.
 */
typedef struct
{ 
	key_t record_key; //!< The record key.
  uint8_t crc8;       // < The
	length_t length;    //!< The length of the record
} minfds_record_t;

/**@brief Functions provided by the API implementation. */
typedef struct
{
  /**@brief flashдp_dstΪĿַp_srcΪԴַsizewordsΪҪд֣32λݣֽڣĳȣдɹҪ0дʧֵܷ*/
  flash_OPER_Result (*flash_write)(uint32_t *p_dst, uint32_t const *p_src, uint32_t sizewords);
  /**@brief page_addrΪҪ׵ַ ɹҪ0ʧֵܷ*/
  flash_OPER_Result (*flash_page_erase)(uint32_t page_addr);
} const flish_api_t;

typedef struct
{
  uint32_t start_addr;
  uint32_t end_addr;
  unsigned short record_count;
  uint32_t free_capacity;
  uint32_t duty_capacity;
  flish_api_t api;
} minfds_t;

/**@brief   FDS return values.
 */
enum
{
  MINIFDS_SUCCESS = 0,           //!< Error. 
  MINIFDS_ERR_NOT_INITIALIZED,   //!< Error. The module has not been initialized.
  MINIFDS_ERR_NOT_FREE_CAPACITY, //!< Error. The ռ䲻>
  MINIFDS_ERR_NOT_FIND_RECORD,   //!< Error. The ûҵ¼>
  MINIFDS_ERR_BUSY,              //!< Error. The æ>
  MINIFDS_ERR_API_FAIL,          //!< Error. The ײӿڱ>
  MINIFDS_ERR_REPEAT_KEY,        //!< Error. The keyֵ󣬲ʹ0>
  MINIFDS_ERR_HARDWARE,					 //!< Error. The flashⲿı>
  MINIFDS_ERR_PARAMETER,				 //!< Error. The >
	MINIFDS_ERR_REPEAT_LENGTH,
};

//----------------------------------------------------------------

/**@brief startaddr  endaddr Ϊ׵ַ
 */
#define DEFINE_MINIINF(name, startaddr, endaddr, fun_write, fun_erase) \
  minfds_t name = {.start_addr = startaddr,                            \
                   .end_addr = endaddr,                                \
                   .record_count = 0,                                  \
                   .free_capacity = endaddr - startaddr,               \
                   .api.flash_write = fun_write,                       \
                   .api.flash_page_erase = fun_erase};

char minfds_record_add(minfds_t *p_minifds, key_t key, uint8_t *data, length_t length);

char minfds_record_find(minfds_t *p_minifds, key_t key, uint8_t *data, length_t *length);
									 
char minfds_record_get_length(minfds_t *p_minifds, key_t key, length_t *length);
									 
char minfds_record_find_mem(minfds_t *p_minifds, uint8_t *mem, int mem_len,key_t * key);

char minfds_record_update(minfds_t *p_minifds, key_t key, uint8_t *data, length_t length);

char minfds_record_delete(minfds_t *p_minifds, key_t key);

char minfds_record_delete_all(minfds_t *p_minifds);

char minfds_init(minfds_t *p_minifds);

char minfds_format(minfds_t *p_minifds);

unsigned short minfds_record_count_get(minfds_t *p_minifds);
									 
char record_index(minfds_t *p_minifds,minfds_record_t *record,uint8_t * data,int index);
//----------------------------------------------------------------


char pm_add(minfds_t *p_minifds,const char * key,int key_len,uint8_t *data, const length_t length);

char pm_find(minfds_t *p_minifds,const char * key,int key_len, uint8_t *data,length_t *length);

char pm_get_length(minfds_t *p_minifds, const char * key,int key_len, length_t *length);

char pm_update(minfds_t *p_minifds, const char * key,int key_len, uint8_t *data, length_t length);

char pm_delete(minfds_t *p_minifds, const char * key,int key_len);

#endif
